@@ -710,22 +710,37 @@ gtk_clutter_embed_style_updated (GtkWidget *widget)
    * a GtkClutterEmbed will not look completely alien
    */
   clutter_settings = clutter_settings_get_default ();
-  g_object_set (G_OBJECT (clutter_settings),
-                "font-name", font_name,
-                "double-click-time", double_click_time,
-                "double-click-distance", double_click_distance,
-#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11) 
-                "font-antialias", xft_antialias,
-                "font-dpi", xft_dpi,
-                "font-hinting", xft_hinting,
-                "font-hint-style", xft_hintstyle,
-                "font-subpixel-order", xft_rgba,
-#endif
-                NULL);
 
 #if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
-  g_free (xft_hintstyle);
-  g_free (xft_rgba);
+  if (GDK_IS_X11_SCREEN (screen))
+    {
+      g_object_set (G_OBJECT (clutter_settings),
+                    "font-name", font_name,
+                    "double-click-time", double_click_time,
+                    "double-click-distance", double_click_distance,
+                    "font-antialias", xft_antialias,
+                    "font-dpi", xft_dpi,
+                    "font-hinting", xft_hinting,
+                    "font-hint-style", xft_hintstyle,
+                    "font-subpixel-order", xft_rgba,
+                    NULL);
+    }
+  else
+#endif
+    {
+      g_object_set (G_OBJECT (clutter_settings),
+                    "font-name", font_name,
+                    "double-click-time", double_click_time,
+                    "double-click-distance", double_click_distance,
+                    NULL);
+    }
+
+#if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
+  if (GDK_IS_X11_SCREEN (screen))
+    {
+      g_free (xft_hintstyle);
+      g_free (xft_rgba);
+    }
 #endif
 
   g_free (font_name);
@@ -826,18 +841,6 @@ gtk_clutter_embed_event (GtkWidget *widget,
   return FALSE;
 }
 
-static void
-gtk_clutter_embed_state_flags_changed (GtkWidget *widget,
-                                       GtkStateFlags prev_state_flags)
-{
-  if (GTK_WIDGET_CLASS (gtk_clutter_embed_parent_class)->state_flags_changed != NULL)
-    GTK_WIDGET_CLASS (gtk_clutter_embed_parent_class)->state_flags_changed (widget, prev_state_flags);
-
-  gtk_container_forall (GTK_CONTAINER (widget),
-                        (GtkCallback) gtk_widget_queue_draw,
-                        NULL);
-}
-
 static void
 gtk_clutter_embed_set_property (GObject       *gobject,
                                 guint          prop_id,
@@ -906,7 +909,6 @@ gtk_clutter_embed_class_init (GtkClutterEmbedClass *klass)
   widget_class->key_press_event = gtk_clutter_embed_key_event;
   widget_class->key_release_event = gtk_clutter_embed_key_event;
   widget_class->event = gtk_clutter_embed_event;
-  widget_class->state_flags_changed = gtk_clutter_embed_state_flags_changed;
   widget_class->get_request_mode = gtk_clutter_embed_get_request_mode;
   widget_class->get_preferred_width = gtk_clutter_embed_get_preferred_width;
   widget_class->get_preferred_height = gtk_clutter_embed_get_preferred_height;
