@@ -76,9 +76,6 @@ post_parse_hook (GOptionContext  *context,
   if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11) &&
       GDK_IS_X11_DISPLAY (display))
     {
-      /* we need XI enabled because GTK will enable it by default as well */
-      clutter_x11_enable_xinput ();
-
       /* enable ARGB visuals by default for Clutter */
       clutter_x11_set_use_argb_visual (TRUE);
 
@@ -185,23 +182,31 @@ gtk_clutter_init (int    *argc,
     return CLUTTER_INIT_ERROR_UNKNOWN;
 
 #if defined(CLUTTER_WINDOWING_GDK)
-  clutter_gdk_disable_event_retrieval ();
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_GDK))
+    clutter_gdk_disable_event_retrieval ();
 #endif
 
 #if defined(GDK_WINDOWING_X11) && defined(CLUTTER_WINDOWING_X11)
-  clutter_x11_enable_xinput ();
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11))
+    {
+      clutter_x11_set_use_argb_visual (TRUE);
 
-  clutter_x11_set_use_argb_visual (TRUE);
+      clutter_x11_set_display (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 
-  clutter_x11_set_display (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-
-  clutter_x11_disable_event_retrieval ();
+      clutter_x11_disable_event_retrieval ();
+    }
 #endif
 
 #if defined(CLUTTER_WINDOWING_WIN32)
-  clutter_win32_disable_event_retrieval ();
+  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_WIN32))
+    clutter_win32_disable_event_retrieval ();
 #endif
 
+  /* We disable clutter accessibility support in order to not
+   * interfere with gtk accessibility support.
+   */
+  clutter_disable_accessibility ();
+
   return clutter_init (argc, argv);
 }
 
